﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Thrift.Protocol;
using Thrift.Transport;

namespace HbaseThriftTutorial
{
    class Examples
    {
        static string serverHostName = "your_server_address_here";
        static int port = 9090;

        public static void GetExample()
        {
            using (var socket = new TSocket(serverHostName, port))
            using (var transport = new TBufferedTransport(socket))
            using (var protocol = new TBinaryProtocol(transport))
            using (var client = new Hbase.Client(protocol))
            {
                byte[] tableName = "tutorial:TestTable".GetUTF8Bytes();

                List<byte[]> rows = new List<byte[]>() { "id_2".GetUTF8Bytes(), "id_3".GetUTF8Bytes(), "id_4".GetUTF8Bytes() };

                List<TRowResult> result = client.getRows(tableName, rows, new Dictionary<byte[], byte[]>());
            }

            //Do something with result
        }

        public static void ScanExample()
        {
            using (var socket = new TSocket(serverHostName, port))
            using (var transport = new TBufferedTransport(socket))
            using (var protocol = new TBinaryProtocol(transport))
            using (var client = new Hbase.Client(protocol))
            {
                TScan tscan = new TScan();
                tscan.Columns = new List<byte[]>() { "family1".GetUTF8Bytes(), "family2".GetUTF8Bytes() };
                tscan.StartRow = "id_5".GetUTF8Bytes();
                tscan.StopRow = "id_23".GetUTF8Bytes();

                byte[] tableName = "tutorial:TestTable".GetUTF8Bytes();

                int scannerId = client.scannerOpenWithScan(tableName, tscan, new Dictionary<byte[], byte[]>());

                int scannerBatchSize = 100;
                List<TRowResult> result = null;

                do
                {
                    result = client.scannerGetList(scannerId, scannerBatchSize);

                    //Do something with result
                }
                while (result.Count == scannerBatchSize);
            }
        }

        public static void FilterExample()
        {
            using (var socket = new TSocket(serverHostName, port))
            using (var transport = new TBufferedTransport(socket))
            using (var protocol = new TBinaryProtocol(transport))
            using (var client = new Hbase.Client(protocol))
            {
                TScan tscan = new TScan();
                tscan.FilterString = Helpers.CreateSingleColumnValueFilter("col_family", "column_name", "=", "filter_value").GetUTF8Bytes();

                byte[] tableName = "tutorial:TestTable".GetUTF8Bytes();

                int scannerId = client.scannerOpenWithScan(tableName, tscan, new Dictionary<byte[], byte[]>());

                int scannerBatchSize = 100;
                List<TRowResult> result = null;

                do
                {
                    result = client.scannerGetList(scannerId, scannerBatchSize);

                    //Do something with result
                }
                while (result.Count == scannerBatchSize);
            }
        }
    }
}
