﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HbaseThriftTutorial
{
    static class Extensions
    {
        public static byte[] GetUTF8Bytes(this string @this)
        {
            return Encoding.UTF8.GetBytes(@this);
        }

        public static string GetUTF8String(this byte[] @this)
        {
            return Encoding.UTF8.GetString(@this);
        }

        public static DateTime ToDateTime(this long @this)
        {
            return new DateTime(@this * 10000 + 621355968000000000);
        }

        public static long TicksJava(this DateTime @this)
        {
            return (@this.Ticks - 621355968000000000) / 10000;
        }

        public static byte[] GetColumnValue(this TRowResult @this, string columnName)
        {
            return @this.Columns.Where(c => c.Key.GetUTF8String() == columnName).Single().Value.Value;
        }

        public static string GetColumnValueAsString(this TRowResult @this, string columnName)
        {
            return @this.GetColumnValue(columnName).GetUTF8String();
        }

        public static void ToFile(this List<TRowResult> @this, string directory)
        {
            if (!Directory.Exists(directory))
                Directory.CreateDirectory(directory);

            var resultsKVP = @this.Select(r => new { Id = r.Row.GetUTF8String(), Columns = r.Columns.Select(c => new { Key = c.Key.GetUTF8String(), Value = c.Value.Value.GetUTF8String() }) }).ToList();

            foreach (var item in resultsKVP)
            {
                using (FileStream fs = new FileStream(Path.Combine(directory, item.Id + ".rowDetails"), FileMode.Create))
                using (StreamWriter sw = new StreamWriter(fs))
                {
                    foreach (var kvp in item.Columns)
                        sw.WriteLine(string.Format("{0}: {1}", kvp.Key, kvp.Value));
                }
            }
        }

        public static List<string[]> ToReadableList(this List<TRowResult> @this)
        {
            List<string[]> values = new List<string[]>();

            for (int i = 0; i < @this.Count; i++)
            {
                string[] vals = new string[@this[i].Columns.Count];

                int j = 0;

                foreach (var column in @this[i].Columns)
                {
                    vals[j++] = column.Value.Value.GetUTF8String();
                }

                values.Add(vals);
            }

            return values;
        }

    }
}
