﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Thrift.Protocol;
using Thrift.Transport;

namespace HbaseThriftTutorial
{
    class HBaseScannerProxy
    {
        [ThreadStatic]
        protected static int? scannerId = null;

        [ThreadStatic]
        protected static Hbase.Client hbClient = null;
        [ThreadStatic]
        protected static TSocket socket = null;
        [ThreadStatic]
        protected static TBufferedTransport transport = null;
        [ThreadStatic]
        protected static TBinaryProtocol protocol = null;
        
        protected bool ConnectToHBase()
        {
            string serverHostName = "get_host_from_your_config";
            int serverPort = 9090;

            try
            {
                socket = new TSocket(serverHostName, serverPort);
                transport = new TBufferedTransport(socket);
                protocol = new TBinaryProtocol(transport);
                hbClient = new Hbase.Client(protocol);

                hbClient.InputProtocol.Transport.Open();

                return true;
            }
            catch (Exception ex)
            {
                //Log exception

                hbClient = null;

                return false;
            }
        }

        public void CloseClient()
        {
            try
            {
                if (hbClient != null)
                    hbClient.Dispose();
            }
            catch (Exception ex)
            {
                //Log exception
            }
            finally
            {
                hbClient = null;
            }

            try
            {
                if (protocol != null)
                    protocol.Dispose();
            }
            catch (Exception ex)
            {
                //Log exception
            }

            try
            {
                if (transport != null)
                {
                    if (transport.IsOpen)
                        transport.Close();

                    transport.Dispose();
                }
            }
            catch (Exception ex)
            {
                //Log exception
            }

            try
            {
                if (socket != null)
                {
                    if (socket.IsOpen)
                        socket.Close();

                    socket.Dispose();
                }
            }
            catch (Exception ex)
            {
                //Log exception
            }
        }

        public void CloseScanner()
        {
            try
            {
                if (hbClient == null)
                    return;

                if (!scannerId.HasValue)
                    return;

                hbClient.scannerClose(scannerId.Value);
            }
            catch (Exception ex)
            {
                //Log exception
            }
            finally
            {
                scannerId = null;
            }
        }
        
        protected bool PrepareScanner(byte[] startRow, byte[] tableName)
        {
            if (hbClient == null && !ConnectToHBase())
                return false;

            try
            {
                //Prepare scanner
                TScan tscan = new TScan();

                if (startRow != null && startRow.Length > 0)
                    tscan.StartRow = startRow;

                scannerId = hbClient.scannerOpenWithScan(tableName, tscan, new Dictionary<byte[], byte[]>());

                return true;
            }
            catch (Exception ex)
            {
                //Log exception

                scannerId = null;

                return false;
            }
        }
        
        public List<TRowResult> GetScannerBatch(int batchSize, byte[] lastSuccessfullyRetrievedRow, byte[] tableName)
        {
            try
            {
                //Prepare connection and scanner
                if (hbClient == null && !ConnectToHBase())
                    throw new Exception("There isn't any active hbClient");

                if (!scannerId.HasValue && !PrepareScanner(lastSuccessfullyRetrievedRow, tableName))
                    throw new Exception("There isn't any active scanner");

                //Get results
                List<TRowResult> results = hbClient.scannerGetList(scannerId.Value, batchSize);

                //Return results
                return results;
            }
            catch (Exception ex)
            {
                //Log exception

                //Try to close current connection
                CloseScanner();
                CloseClient();

                //Return null to make program do next loop iteration
                return null;
            }
        }
    }
}
